@@ -306,37 +306,83 @@ static int cache_check (const char *type, const value_list_t *vl)
 static int write_part_values (char **ret_buffer, int *ret_buffer_len,
 		const data_set_t *ds, const value_list_t *vl)
 {
-	part_values_t pv;
+	char *packet_ptr;
+	int packet_len;
+	int num_values;
+
+	part_header_t pkg_ph;
+	uint16_t      pkg_num_values;
+	uint8_t      *pkg_values_types;
+	value_t      *pkg_values;
+
+	int offset;
 	int i;
 
-	i = 6 + (9 * vl->values_len);
-	if (*ret_buffer_len < i)
+	num_values = vl->values_len;
+	packet_len = sizeof (part_header_t) + sizeof (uint16_t)
+		+ (num_values * sizeof (uint8_t))
+		+ (num_values * sizeof (value_t));
+
+	if (*ret_buffer_len < packet_len)
 		return (-1);
-	*ret_buffer_len -= i;
 
-	pv.head = (part_header_t *) *ret_buffer;
-	pv.num_values = (uint16_t *) (pv.head + 1);
-	pv.values_types = (uint8_t *) (pv.num_values + 1);
-	pv.values = (value_t *) (pv.values_types + vl->values_len);
-	*ret_buffer = (void *) (pv.values + vl->values_len);
+	pkg_values_types = (uint8_t *) malloc (num_values * sizeof (uint8_t));
+	if (pkg_values_types == NULL)
+	{
+		ERROR ("network plugin: write_part_values: malloc failed.");
+		return (-1);
+	}
 
-	pv.head->type = htons (TYPE_VALUES);
-	pv.head->length = htons (6 + (9 * vl->values_len));
-	*pv.num_values = htons ((uint16_t) vl->values_len);
-	
-	for (i = 0; i < vl->values_len; i++)
+	pkg_values = (value_t *) malloc (num_values * sizeof (value_t));
+	if (pkg_values == NULL)
+	{
+		free (pkg_values_types);
+		ERROR ("network plugin: write_part_values: malloc failed.");
+		return (-1);
+	}
+
+	pkg_ph.type = htons (TYPE_VALUES);
+	pkg_ph.length = htons (packet_len);
+
+	pkg_num_values = htons ((uint16_t) vl->values_len);
+
+	for (i = 0; i < num_values; i++)
 	{
 		if (ds->ds[i].type == DS_TYPE_COUNTER)
 		{
-			pv.values_types[i] = DS_TYPE_COUNTER;
-			pv.values[i].counter = htonll (vl->values[i].counter);
+			pkg_values_types[i] = DS_TYPE_COUNTER;
+			pkg_values[i].counter = htonll (vl->values[i].counter);
 		}
 		else
 		{
-			pv.values_types[i] = DS_TYPE_GAUGE;
-			pv.values[i].gauge = vl->values[i].gauge;
+			pkg_values_types[i] = DS_TYPE_GAUGE;
+			pkg_values[i].gauge = htond (vl->values[i].gauge);
 		}
-	} /* for (values) */
+	}
+
+	/*
+	 * Use `memcpy' to write everything to the buffer, because the pointer
+	 * may be unaligned and some architectures, such as SPARC, can't handle
+	 * that.
+	 */
+	packet_ptr = *ret_buffer;
+	offset = 0;
+	memcpy (packet_ptr + offset, &pkg_ph, sizeof (pkg_ph));
+	offset += sizeof (pkg_ph);
+	memcpy (packet_ptr + offset, &pkg_num_values, sizeof (pkg_num_values));
+	offset += sizeof (pkg_num_values);
+	memcpy (packet_ptr + offset, pkg_values_types, num_values * sizeof (uint8_t));
+	offset += num_values * sizeof (uint8_t);
+	memcpy (packet_ptr + offset, pkg_values, num_values * sizeof (value_t));
+	offset += num_values * sizeof (value_t);
+
+	assert (offset == packet_len);
+
+	*ret_buffer = packet_ptr + packet_len;
+	*ret_buffer_len -= packet_len;
+
+	free (pkg_values_types);
+	free (pkg_values);
 
 	return (0);
 } /* int write_part_values */
@@ -344,20 +390,34 @@ static int write_part_values (char **ret_buffer, int *ret_buffer_len,
 static int write_part_number (char **ret_buffer, int *ret_buffer_len,
 		int type, uint64_t value)
 {
-	part_number_t pn;
+	char *packet_ptr;
+	int packet_len;
 
-	if (*ret_buffer_len < 12)
+	part_header_t pkg_head;
+	uint64_t pkg_value;
+	
+	int offset;
+
+	packet_len = sizeof (pkg_head) + sizeof (pkg_value);
+
+	if (*ret_buffer_len < packet_len)
 		return (-1);
 
-	pn.head = (part_header_t *) *ret_buffer;
-	pn.value = (uint64_t *) (pn.head + 1);
+	pkg_head.type = htons (type);
+	pkg_head.length = htons (packet_len);
+	pkg_value = htonll (value);
 
-	pn.head->type = htons (type);
-	pn.head->length = htons (12);
-	*pn.value = htonll (value);
+	packet_ptr = *ret_buffer;
+	offset = 0;
+	memcpy (packet_ptr + offset, &pkg_head, sizeof (pkg_head));
+	offset += sizeof (pkg_head);
+	memcpy (packet_ptr + offset, &pkg_value, sizeof (pkg_value));
+	offset += sizeof (pkg_value);
 
-	*ret_buffer = (char *) (pn.value + 1);
-	*ret_buffer_len -= 12;
+	assert (offset == packet_len);
+
+	*ret_buffer = packet_ptr + packet_len;
+	*ret_buffer_len -= packet_len;
 
 	return (0);
 } /* int write_part_number */
@@ -365,23 +425,36 @@ static int write_part_number (char **ret_buffer, int *ret_buffer_len,
 static int write_part_string (char **ret_buffer, int *ret_buffer_len,
 		int type, const char *str, int str_len)
 {
-	part_string_t ps;
-	int len;
+	char *buffer;
+	int buffer_len;
 
-	len = 4 + str_len + 1;
-	if (*ret_buffer_len < len)
+	uint16_t pkg_type;
+	uint16_t pkg_length;
+
+	int offset;
+
+	buffer_len = 2 * sizeof (uint16_t) + str_len + 1;
+	if (*ret_buffer_len < buffer_len)
 		return (-1);
-	*ret_buffer_len -= len;
 
-	ps.head = (part_header_t *) *ret_buffer;
-	ps.value = (char *) (ps.head + 1);
+	pkg_type = htons (type);
+	pkg_length = htons (buffer_len);
 
-	ps.head->type = htons ((uint16_t) type);
-	ps.head->length = htons ((uint16_t) str_len + 5);
-	if (str_len > 0)
-		memcpy (ps.value, str, str_len);
-	ps.value[str_len] = '\0';
-	*ret_buffer = (void *) (ps.value + (str_len + 1));
+	buffer = *ret_buffer;
+	offset = 0;
+	memcpy (buffer + offset, (void *) &pkg_type, sizeof (pkg_type));
+	offset += sizeof (pkg_type);
+	memcpy (buffer + offset, (void *) &pkg_length, sizeof (pkg_length));
+	offset += sizeof (pkg_length);
+	memcpy (buffer + offset, str, str_len);
+	offset += str_len;
+	memset (buffer + offset, '\0', 1);
+	offset += 1;
+
+	assert (offset == buffer_len);
+
+	*ret_buffer = buffer + buffer_len;
+	*ret_buffer_len -= buffer_len;
 
 	return (0);
 } /* int write_part_string */
@@ -391,12 +464,17 @@ static int parse_part_values (void **ret_buffer, int *ret_buffer_len,
 {
 	char *buffer = *ret_buffer;
 	int   buffer_len = *ret_buffer_len;
-	part_values_t pv;
+
+	uint16_t tmp16;
+	size_t exp_size;
 	int   i;
 
-	uint16_t h_length;
-	uint16_t h_type;
-	uint16_t h_num;
+	uint16_t pkg_length;
+	uint16_t pkg_type;
+	uint16_t pkg_numval;
+
+	uint8_t *pkg_types;
+	value_t *pkg_values;
 
 	if (buffer_len < (15))
 	{
@@ -405,32 +483,69 @@ static int parse_part_values (void **ret_buffer, int *ret_buffer_len,
 		return (-1);
 	}
 
-	pv.head = (part_header_t *) buffer;
-	h_length = ntohs (pv.head->length);
-	h_type = ntohs (pv.head->type);
+	memcpy ((void *) &tmp16, buffer, sizeof (tmp16));
+	buffer += sizeof (tmp16);
+	pkg_type = ntohs (tmp16);
 
-	assert (h_type == TYPE_VALUES);
+	memcpy ((void *) &tmp16, buffer, sizeof (tmp16));
+	buffer += sizeof (tmp16);
+	pkg_length = ntohs (tmp16);
 
-	pv.num_values = (uint16_t *) (pv.head + 1);
-	h_num = ntohs (*pv.num_values);
+	memcpy ((void *) &tmp16, buffer, sizeof (tmp16));
+	buffer += sizeof (tmp16);
+	pkg_numval = ntohs (tmp16);
 
-	if (h_num != ((h_length - 6) / 9))
+	assert (pkg_type == TYPE_VALUES);
+
+	exp_size = 3 * sizeof (uint16_t)
+		+ pkg_numval * (sizeof (uint8_t) + sizeof (value_t));
+	if (buffer_len < exp_size)
 	{
-		DEBUG ("`length' and `num of values' don't match");
+		WARNING ("network plugin: parse_part_values: "
+				"Packet too short: "
+				"Chunk of size %u expected, "
+				"but buffer has only %i bytes left.",
+				(unsigned int) exp_size, buffer_len);
 		return (-1);
 	}
 
-	pv.values_types = (uint8_t *) (pv.num_values + 1);
-	pv.values = (value_t *) (pv.values_types + h_num);
+	if (pkg_length != exp_size)
+	{
+		WARNING ("network plugin: parse_part_values: "
+				"Length and number of values "
+				"in the packet don't match.");
+		return (-1);
+	}
 
-	for (i = 0; i < h_num; i++)
-		if (pv.values_types[i] == DS_TYPE_COUNTER)
-			pv.values[i].counter = ntohll (pv.values[i].counter);
+	pkg_types = (uint8_t *) malloc (pkg_numval * sizeof (uint8_t));
+	pkg_values = (value_t *) malloc (pkg_numval * sizeof (value_t));
+	if ((pkg_types == NULL) || (pkg_values == NULL))
+	{
+		sfree (pkg_types);
+		sfree (pkg_values);
+		ERROR ("network plugin: parse_part_values: malloc failed.");
+		return (-1);
+	}
 
-	*ret_buffer     = (void *) (pv.values + h_num);
-	*ret_buffer_len = buffer_len - h_length;
-	*ret_num_values = h_num;
-	*ret_values     = pv.values;
+	memcpy ((void *) pkg_types, (void *) buffer, pkg_numval * sizeof (uint8_t));
+	buffer += pkg_numval * sizeof (uint8_t);
+	memcpy ((void *) pkg_values, (void *) buffer, pkg_numval * sizeof (value_t));
+	buffer += pkg_numval * sizeof (value_t);
+
+	for (i = 0; i < pkg_numval; i++)
+	{
+		if (pkg_types[i] == DS_TYPE_COUNTER)
+			pkg_values[i].counter = ntohll (pkg_values[i].counter);
+		else if (pkg_types[i] == DS_TYPE_GAUGE)
+			pkg_values[i].gauge = ntohd (pkg_values[i].gauge);
+	}
+
+	*ret_buffer     = buffer;
+	*ret_buffer_len = buffer_len - pkg_length;
+	*ret_num_values = pkg_numval;
+	*ret_values     = pkg_values;
+
+	sfree (pkg_types);
 
 	return (0);
 } /* int parse_part_values */
@@ -438,21 +553,40 @@ static int parse_part_values (void **ret_buffer, int *ret_buffer_len,
 static int parse_part_number (void **ret_buffer, int *ret_buffer_len,
 		uint64_t *value)
 {
-	part_number_t pn;
-	uint16_t len;
+	char *buffer = *ret_buffer;
+	int buffer_len = *ret_buffer_len;
 
-	pn.head = (part_header_t *) *ret_buffer;
-	pn.value = (uint64_t *) (pn.head + 1);
+	uint16_t tmp16;
+	uint64_t tmp64;
+	size_t exp_size = 2 * sizeof (uint16_t) + sizeof (uint64_t);
 
-	len = ntohs (pn.head->length);
-	if (len != 12)
+	uint16_t pkg_length;
+	uint16_t pkg_type;
+
+	if (buffer_len < exp_size)
+	{
+		WARNING ("network plugin: parse_part_number: "
+				"Packet too short: "
+				"Chunk of size %u expected, "
+				"but buffer has only %i bytes left.",
+				(unsigned int) exp_size, buffer_len);
 		return (-1);
-	if (len > *ret_buffer_len)
-		return (-1);
-	*value = ntohll (*pn.value);
+	}
 
-	*ret_buffer = (void *) (pn.value + 1);
-	*ret_buffer_len -= len;
+	memcpy ((void *) &tmp16, buffer, sizeof (tmp16));
+	buffer += sizeof (tmp16);
+	pkg_type = ntohs (tmp16);
+
+	memcpy ((void *) &tmp16, buffer, sizeof (tmp16));
+	buffer += sizeof (tmp16);
+	pkg_length = ntohs (tmp16);
+
+	memcpy ((void *) &tmp64, buffer, sizeof (tmp64));
+	buffer += sizeof (tmp64);
+	*value = ntohll (tmp64);
+
+	*ret_buffer = buffer;
+	*ret_buffer_len = buffer_len - pkg_length;
 
 	return (0);
 } /* int parse_part_number */
@@ -462,61 +596,85 @@ static int parse_part_string (void **ret_buffer, int *ret_buffer_len,
 {
 	char *buffer = *ret_buffer;
 	int   buffer_len = *ret_buffer_len;
-	part_string_t ps;
 
-	uint16_t h_length;
-	uint16_t h_type;
+	uint16_t tmp16;
+	size_t header_size = 2 * sizeof (uint16_t);
 
-	DEBUG ("network plugin: parse_part_string: ret_buffer = %p;"
-			" ret_buffer_len = %i; output = %p; output_len = %i;",
-			*ret_buffer, *ret_buffer_len,
-			(void *) output, output_len);
+	uint16_t pkg_length;
+	uint16_t pkg_type;
 
-	ps.head = (part_header_t *) buffer;
-
-	h_length = ntohs (ps.head->length);
-	h_type = ntohs (ps.head->type);
-
-	DEBUG ("network plugin: parse_part_string: length = %hu; type = %hu;",
-			h_length, h_type);
-
-	if (buffer_len < h_length)
+	if (buffer_len < header_size)
 	{
-		DEBUG ("packet is too short");
-		return (-1);
-	}
-	assert ((h_type == TYPE_HOST)
-			|| (h_type == TYPE_PLUGIN)
-			|| (h_type == TYPE_PLUGIN_INSTANCE)
-			|| (h_type == TYPE_TYPE)
-			|| (h_type == TYPE_TYPE_INSTANCE)
-			|| (h_type == TYPE_MESSAGE));
-
-	ps.value = buffer + 4;
-	if (ps.value[h_length - 5] != '\0')
-	{
-		DEBUG ("String does not end with a nullbyte");
+		WARNING ("network plugin: parse_part_string: "
+				"Packet too short: "
+				"Chunk of at least size %u expected, "
+				"but buffer has only %i bytes left.",
+				(unsigned int) header_size, buffer_len);
 		return (-1);
 	}
 
-	if (output_len < (h_length - 4))
+	memcpy ((void *) &tmp16, buffer, sizeof (tmp16));
+	buffer += sizeof (tmp16);
+	pkg_type = ntohs (tmp16);
+
+	memcpy ((void *) &tmp16, buffer, sizeof (tmp16));
+	buffer += sizeof (tmp16);
+	pkg_length = ntohs (tmp16);
+
+	/* Check that packet fits in the input buffer */
+	if (pkg_length > buffer_len)
 	{
-		DEBUG ("output buffer is too small");
+		WARNING ("network plugin: parse_part_string: "
+				"Packet too big: "
+				"Chunk of size %hu received, "
+				"but buffer has only %i bytes left.",
+				pkg_length, buffer_len);
 		return (-1);
 	}
-	strcpy (output, ps.value);
 
-	DEBUG ("network plugin: parse_part_string: output = %s", output);
+	/* Check that pkg_length is in the valid range */
+	if (pkg_length <= header_size)
+	{
+		WARNING ("network plugin: parse_part_string: "
+				"Packet too short: "
+				"Header claims this packet is only %hu "
+				"bytes long.", pkg_length);
+		return (-1);
+	}
 
-	*ret_buffer = (void *) (buffer + h_length);
-	*ret_buffer_len = buffer_len - h_length;
+	/* Check that the package data fits into the output buffer.
+	 * The previous if-statement ensures that:
+	 * `pkg_length > header_size' */
+	if ((pkg_length - header_size) > output_len)
+	{
+		WARNING ("network plugin: parse_part_string: "
+				"Output buffer too small.");
+		return (-1);
+	}
+
+	/* All sanity checks successfull, let's copy the data over */
+	output_len = pkg_length - header_size;
+	memcpy ((void *) output, (void *) buffer, output_len);
+	buffer += output_len;
+
+	/* For some very weird reason '\0' doesn't do the trick on SPARC in
+	 * this statement. */
+	if (output[output_len - 1] != 0)
+	{
+		WARNING ("network plugin: parse_part_string: "
+				"Received string does not end "
+				"with a NULL-byte.");
+		return (-1);
+	}
+
+	*ret_buffer = buffer;
+	*ret_buffer_len = buffer_len - pkg_length;
 
 	return (0);
 } /* int parse_part_string */
 
 static int parse_packet (void *buffer, int buffer_len)
 {
-	part_header_t *header;
 	int status;
 
 	value_list_t vl = VALUE_LIST_INIT;
@@ -534,24 +692,32 @@ static int parse_packet (void *buffer, int buffer_len)
 	while ((status == 0) && (0 < buffer_len)
 			&& ((unsigned int) buffer_len > sizeof (part_header_t)))
 	{
-		header = (part_header_t *) buffer;
+		uint16_t pkg_length;
+		uint16_t pkg_type;
 
-		if (ntohs (header->length) > buffer_len)
+		memcpy ((void *) &pkg_type,
+				(void *) buffer,
+				sizeof (pkg_type));
+		memcpy ((void *) &pkg_length,
+				(void *) (buffer + sizeof (pkg_type)),
+				sizeof (pkg_length));
+
+		pkg_length = ntohs (pkg_length);
+		pkg_type = ntohs (pkg_type);
+
+		if (pkg_length > buffer_len)
 			break;
-		/* Assure that this loop terminates eventually */
-		if (ntohs (header->length) < 4)
+		/* Ensure that this loop terminates eventually */
+		if (pkg_length < (2 * sizeof (uint16_t)))
 			break;
 
-		if (ntohs (header->type) == TYPE_VALUES)
+		if (pkg_type == TYPE_VALUES)
 		{
 			status = parse_part_values (&buffer, &buffer_len,
 					&vl.values, &vl.values_len);
 
 			if (status != 0)
-			{
-				DEBUG ("parse_part_values failed.");
 				break;
-			}
 
 			if ((vl.time > 0)
 					&& (strlen (vl.host) > 0)
@@ -559,8 +725,6 @@ static int parse_packet (void *buffer, int buffer_len)
 					&& (strlen (type) > 0)
 					&& (cache_check (type, &vl) == 0))
 			{
-				DEBUG ("network plugin: parse_packet:"
-						" dispatching values");
 				plugin_dispatch_values (type, &vl);
 			}
 			else
@@ -568,89 +732,85 @@ static int parse_packet (void *buffer, int buffer_len)
 				DEBUG ("network plugin: parse_packet:"
 						" NOT dispatching values");
 			}
+
+			sfree (vl.values);
 		}
-		else if (ntohs (header->type) == TYPE_TIME)
+		else if (pkg_type == TYPE_TIME)
 		{
 			uint64_t tmp = 0;
-			status = parse_part_number (&buffer, &buffer_len, &tmp);
+			status = parse_part_number (&buffer, &buffer_len,
+					&tmp);
 			if (status == 0)
 			{
 				vl.time = (time_t) tmp;
 				n.time = (time_t) tmp;
 			}
 		}
-		else if (ntohs (header->type) == TYPE_INTERVAL)
+		else if (pkg_type == TYPE_INTERVAL)
 		{
 			uint64_t tmp = 0;
-			status = parse_part_number (&buffer, &buffer_len, &tmp);
+			status = parse_part_number (&buffer, &buffer_len,
+					&tmp);
 			if (status == 0)
 				vl.interval = (int) tmp;
 		}
-		else if (ntohs (header->type) == TYPE_HOST)
+		else if (pkg_type == TYPE_HOST)
 		{
 			status = parse_part_string (&buffer, &buffer_len,
 					vl.host, sizeof (vl.host));
-			strncpy (n.host, vl.host, sizeof (n.host));
-			n.host[sizeof (n.host) - 1] = '\0';
-			DEBUG ("network plugin: parse_packet: vl.host = %s",
-					vl.host);
+			if (status == 0)
+				sstrncpy (n.host, vl.host, sizeof (n.host));
 		}
-		else if (ntohs (header->type) == TYPE_PLUGIN)
+		else if (pkg_type == TYPE_PLUGIN)
 		{
 			status = parse_part_string (&buffer, &buffer_len,
 					vl.plugin, sizeof (vl.plugin));
-			strncpy (n.plugin, vl.plugin, sizeof (n.plugin));
-			n.plugin[sizeof (n.plugin) - 1] = '\0';
-			DEBUG ("network plugin: parse_packet: vl.plugin = %s",
-					vl.plugin);
+			if (status == 0)
+				sstrncpy (n.plugin, vl.plugin,
+						sizeof (n.plugin));
 		}
-		else if (ntohs (header->type) == TYPE_PLUGIN_INSTANCE)
+		else if (pkg_type == TYPE_PLUGIN_INSTANCE)
 		{
 			status = parse_part_string (&buffer, &buffer_len,
 					vl.plugin_instance,
 					sizeof (vl.plugin_instance));
-			strncpy (n.plugin_instance, vl.plugin_instance,
-					sizeof (n.plugin_instance));
-			n.plugin_instance[sizeof (n.plugin_instance) - 1] = '\0';
-			DEBUG ("network plugin: parse_packet: "
-					"vl.plugin_instance = %s",
-					vl.plugin_instance);
+			if (status == 0)
+				sstrncpy (n.plugin_instance,
+						vl.plugin_instance,
+						sizeof (n.plugin_instance));
 		}
-		else if (ntohs (header->type) == TYPE_TYPE)
+		else if (pkg_type == TYPE_TYPE)
 		{
 			status = parse_part_string (&buffer, &buffer_len,
 					type, sizeof (type));
-			strncpy (n.type, type, sizeof (n.type));
-			n.type[sizeof (n.type) - 1] = '\0';
-			DEBUG ("network plugin: parse_packet: type = %s",
-					type);
+			if (status == 0)
+				sstrncpy (n.type, type, sizeof (n.type));
 		}
-		else if (ntohs (header->type) == TYPE_TYPE_INSTANCE)
+		else if (pkg_type == TYPE_TYPE_INSTANCE)
 		{
 			status = parse_part_string (&buffer, &buffer_len,
 					vl.type_instance,
 					sizeof (vl.type_instance));
-			strncpy (n.type_instance, vl.type_instance,
-					sizeof (n.type_instance));
-			n.type_instance[sizeof (n.type_instance) - 1] = '\0';
-			DEBUG ("network plugin: parse_packet: "
-					"vl.type_instance = %s",
-					vl.type_instance);
+			if (status == 0)
+				sstrncpy (n.type_instance, vl.type_instance,
+						sizeof (n.type_instance));
 		}
-		else if (ntohs (header->type) == TYPE_MESSAGE)
+		else if (pkg_type == TYPE_MESSAGE)
 		{
 			status = parse_part_string (&buffer, &buffer_len,
 					n.message, sizeof (n.message));
-			DEBUG ("network plugin: parse_packet: n.message = %s",
-					n.message);
 
-			if ((n.severity != NOTIF_FAILURE)
+			if (status != 0)
+			{
+				/* do nothing */
+			}
+			else if ((n.severity != NOTIF_FAILURE)
 					&& (n.severity != NOTIF_WARNING)
 					&& (n.severity != NOTIF_OKAY))
 			{
 				INFO ("network plugin: "
 						"Ignoring notification with "
-						"unknown severity %s.",
+						"unknown severity %i.",
 						n.severity);
 			}
 			else if (n.time <= 0)
@@ -667,29 +827,26 @@ static int parse_packet (void *buffer, int buffer_len)
 			}
 			else
 			{
-				/*
-				 * TODO: Let this do a separate thread so that
-				 * no packets are lost if this takes too long.
-				 */
 				plugin_dispatch_notification (&n);
 			}
 		}
-		else if (ntohs (header->type) == TYPE_SEVERITY)
+		else if (pkg_type == TYPE_SEVERITY)
 		{
 			uint64_t tmp = 0;
-			status = parse_part_number (&buffer, &buffer_len, &tmp);
+			status = parse_part_number (&buffer, &buffer_len,
+					&tmp);
 			if (status == 0)
 				n.severity = (int) tmp;
 		}
 		else
 		{
 			DEBUG ("network plugin: parse_packet: Unknown part"
-					" type: 0x%0hx", ntohs (header->type));
-			buffer = ((char *) buffer) + ntohs (header->length);
+					" type: 0x%04hx", pkg_type);
+			buffer = ((char *) buffer) + pkg_length;
 		}
 	} /* while (buffer_len > sizeof (part_header_t)) */
 
-	return (0);
+	return (status);
 } /* int parse_packet */
 
 static void free_sockent (sockent_t *se)
@@ -1602,9 +1759,25 @@ static int network_init (void)
 	return (0);
 } /* int network_init */
 
+static int network_flush (int timeout)
+{
+	pthread_mutex_lock (&send_buffer_lock);
+
+	if (((time (NULL) - cache_flush_last) >= timeout)
+			&& (send_buffer_fill > 0))
+	{
+		flush_buffer ();
+	}
+
+	pthread_mutex_unlock (&send_buffer_lock);
+
+	return (0);
+} /* int network_flush */
+
 void module_register (void)
 {
 	plugin_register_config ("network", network_config,
 			config_keys, config_keys_num);
 	plugin_register_init   ("network", network_init);
+	plugin_register_flush   ("network", network_flush);
 } /* void module_register */
